import { fabric } from "fabric";
import {
  clearObjects,
  sortObjects,
  transformHighlightedText,
  checkTextBoxHeight,
} from "../../components/canvas/utils";
import { PaddedHighlightingTextbox } from "../../components/canvas/textbox";
import backgroundURL from "../../assets/template/make_a_wish_tour_social_wide/background.png";
import backgroundURLInverted from "../../assets/template/make_a_wish_banner/background_inverted.png";
import COLORS from "../../colors";

let mainTextBox = null;

let attendeesTextBox = null;
let dateTextBox = null;
let timeTextBox = null;
let locationTextBox = null;

let mainImage = null;
let backgroundImage = null;
let previousBackgroundImageColor = null;

let contractedByTextbox = null;

let mainImageSource = null;

const removeDownEventListener = () => {
  document
    .getElementsByClassName("upper-canvas")[0]
    .removeEventListener("pointerdown", canvasPointerDownEvent);
};

let upEventFunction = null;
let canvasPointerDownEvent = null;
let pointerDownEventAssigned = false;

const redraw = async (canvas, options) => {
  clearObjects(
    [
      mainTextBox,
      contractedByTextbox,
      attendeesTextBox,
      dateTextBox,
      timeTextBox,
      locationTextBox
    ],
    canvas,
  );

  document
    .getElementsByClassName("upper-canvas")[0]
    .removeEventListener("pointerup", upEventFunction);
  document
    .getElementsByClassName("upper-canvas")[0]
    .removeEventListener("pointerout", upEventFunction);
  document
    .getElementsByClassName("upper-canvas")[0]
    .removeEventListener("pointercancel", upEventFunction);

  canvas.preserveObjectStacking = true;

  const textMarginLeft = Math.ceil(canvas.width * 0.1);
  const textMarginRight = Math.ceil(canvas.width * 0.065);

  const topTextMarginRight = Math.ceil(canvas.width * 0.215);

  let mainTextMarginBottom = Math.ceil(canvas.height * 0.47);
  const dateTextMarginBottom = Math.ceil(canvas.height * 0.8);
  const timeTextBoxMarginBottom = Math.ceil(canvas.height * 0.715);
  const locationTextBoxMarginBottom = Math.ceil(canvas.height * 0.43);
  const attendeesTextBoxMarginBottom = Math.ceil(canvas.height * 0.11);

  const mainTextSize = Math.ceil(canvas.height * 0.210);
  const mainTextLineHeight = 1;

  const dateTextSize = Math.ceil(canvas.height * 0.062);
  const timeTextSize = Math.ceil(canvas.height * 0.062);
  const attendeesTextSize = Math.ceil(canvas.height * 0.035);
  const attendeesTextLineHeight = 1;

  const locationTextSize = Math.ceil(canvas.height * 0.055);
  const locationTextLineHeight = 1;

  const contractedByTextSize = Math.ceil(canvas.height * 0.02);
  const contractedByTextMaxWidth = Math.ceil(canvas.width * 0.9);
  const contractedByTextSidesMargin = Math.ceil(canvas.width * 0.03);

  if (options.dateText !== null) {
    /* BEGIN Date text render */

    const dateTextWidth = canvas.width * 5;  // IDFK

    console.log(options);

    const highlightedData = transformHighlightedText(
      options.dateText,
      mainTextSize,
      dateTextWidth,
      "Bebas Neue",
      options.colors.highlightedText.value,
      options.colors.highlight.value,
      { padWhenDiacritics: false, invertHighlight: true },
    );

    dateTextBox = new PaddedHighlightingTextbox(highlightedData.text, {
      width: dateTextWidth,
      left: 0,
      textAlign: "right",
      fontFamily: "Bebas Neue",
      fontSize: dateTextSize,
      fill: options.colors.highlight.value,
      styles: highlightedData.styles,
      selectable: false,
      highlightPadding: canvas.height * 0.003,
      zIndex: 20,
    });

    checkTextBoxHeight(dateTextBox, 1);

    canvas.add(dateTextBox);

    const dateTextBoxTop =
      canvas.height - dateTextBox.height - dateTextMarginBottom;

      dateTextBox.top = dateTextBoxTop - highlightedData.paddingBottom;

    const dateTextBoxLeft =
      canvas.width - dateTextBox.width - topTextMarginRight;

      dateTextBox.left = dateTextBoxLeft;

    canvas.renderAll();

    /* END Date text render */
  }

  if (options.timeText !== null) {
    /* BEGIN Time text render */

    const timeTextWidth = canvas.width;

    const highlightedData = transformHighlightedText(
      options.timeText,
      timeTextSize,
      timeTextWidth,
      "Bebas Neue",
      options.colors.highlightedText.value,
      options.colors.highlight.value,
      { padWhenDiacritics: false, invertHighlight: true },
    );

    timeTextBox = new PaddedHighlightingTextbox(highlightedData.text, {
      width: canvas.width,
      left: 0,
      textAlign: "right",
      fontFamily: "Bebas Neue",
      fontSize: timeTextSize,
      fill: options.colors.baseText.value,
      styles: highlightedData.styles,
      selectable: false,
      highlightPadding: canvas.height * 0.003,
      zIndex: 20,
    });

    checkTextBoxHeight(timeTextBox, 1);

    canvas.add(timeTextBox);

    const timeTextBoxTop =
      canvas.height - timeTextBox.height - timeTextBoxMarginBottom;

    timeTextBox.top = timeTextBoxTop - highlightedData.paddingBottom;

    const timeTextBoxLeft =
      canvas.width - timeTextBox.width - topTextMarginRight;

    timeTextBox.left = timeTextBoxLeft;

    canvas.renderAll();

    /* END Time text render */
  }

  if (options.locationText !== null) {
    /* BEGIN Location text render */

    const locationTextWidth = canvas.width - textMarginLeft - textMarginRight;

    const highlightedData = transformHighlightedText(
      options.locationText,
      locationTextSize,
      locationTextWidth,
      "Bebas Neue",
      options.colors.highlightedText.value,
      options.colors.highlight.value,
      { padWhenDiacritics: false, invertHighlight: true },
    );

    locationTextBox = new PaddedHighlightingTextbox(highlightedData.text, {
      width: canvas.width,
      left: 0,
      textAlign: "right",
      fontFamily: "Bebas Neue",
      fontSize: locationTextSize,
      fill: options.colors.baseText.value,
      styles: highlightedData.styles,
      lineHeight: locationTextLineHeight,
      selectable: false,
      highlightPadding: canvas.height * 0.003,
      zIndex: 20,
    });

    checkTextBoxHeight(locationTextBox, 2);

    canvas.add(locationTextBox);

    canvas.renderAll();

    const locationTextBoxTop =
      canvas.height - locationTextBox.height - locationTextBoxMarginBottom + (locationTextSize * locationTextLineHeight * (locationTextBox.textLines.length - 1));

    locationTextBox.top = locationTextBoxTop - highlightedData.paddingBottom;

    const locationTextBoxLeft =
      canvas.width - locationTextBox.width - textMarginRight;

    locationTextBox.left = locationTextBoxLeft;

    canvas.renderAll();

    /* END Location text render */
  }

  if (options.attendeesText !== null) {
    /* BEGIN Attendees text render */

    const attendeesTextWidth = canvas.width - textMarginLeft - textMarginRight;

    const highlightedData = transformHighlightedText(
      options.attendeesText,
      attendeesTextSize,
      attendeesTextWidth,
      "Roboto Condensed",
      options.colors.highlightedText.value,
      options.colors.highlight.value,
      { padWhenDiacritics: false, invertHighlight: true },
    );

    attendeesTextBox = new PaddedHighlightingTextbox(highlightedData.text, {
      width: canvas.width,
      left: 0,
      textAlign: "right",
      fontFamily: "Roboto Condensed",
      fontSize: attendeesTextSize,
      fill: options.colors.baseText.value,
      styles: highlightedData.styles,
      selectable: false,
      highlightPadding: canvas.height * 0.003,
      zIndex: 20,
    });

    checkTextBoxHeight(attendeesTextBox, 3);

    canvas.add(attendeesTextBox);

    const attendeesTextBoxTop =
      canvas.height - attendeesTextBox.height - attendeesTextBoxMarginBottom + (attendeesTextSize * attendeesTextLineHeight * (attendeesTextBox.textLines.length - 1));

    attendeesTextBox.top = attendeesTextBoxTop - highlightedData.paddingBottom;

    const attendeesTextBoxLeft =
      canvas.width - attendeesTextBox.width - textMarginRight;

      attendeesTextBox.left = attendeesTextBoxLeft;

    canvas.renderAll();

    /* END Attendees text render */
  }

  if (options.mainText !== null) {
    /* BEGIN Background render */

    if (backgroundImage === null || options.colors.background.value != previousBackgroundImageColor.value || !canvas.getObjects().includes(backgroundImage)) {
      backgroundImage = new Image();

      await new Promise((resolve) => {
        backgroundImage.onload = () => {
          resolve();
        };

        if (options.colors.background.value == COLORS.black.value) {
          backgroundImage.src = backgroundURL;
        } else {
          backgroundImage.src = backgroundURLInverted;
        }
      });

      backgroundImage = new fabric.Image(backgroundImage, {
        top: 0,
        left: 0,
        zIndex: 10,
        selectable: false,
      });
      backgroundImage.scaleToHeight(canvas.height);

      previousBackgroundImageColor = options.colors.background;

      canvas.add(backgroundImage);
    }

    /* END Background render */


    /* BEGIN Main text render */

    const mainTextWidth = canvas.width - textMarginLeft - textMarginRight;

    const highlightedData = transformHighlightedText(
      options.mainText,
      mainTextSize,
      mainTextWidth,
      "Bebas Neue",
      options.colors.highlightedText.value,
      options.colors.highlight.value,
      { padWhenDiacritics: false, invertHighlight: true },
    );

    mainTextBox = new PaddedHighlightingTextbox(highlightedData.text, {
      width: canvas.width,
      left: 0,
      textAlign: "right",
      fontFamily: "Bebas Neue",
      fontSize: mainTextSize,
      lineHeight: mainTextLineHeight,
      fill: options.colors.highlight.value,
      styles: highlightedData.styles,
      selectable: false,
      highlightPadding: canvas.height * 0.003,
      zIndex: 20,
    });

    checkTextBoxHeight(mainTextBox, 1);

    canvas.add(mainTextBox);

    const mainTextBoxTop =
      canvas.height - mainTextBox.height - mainTextMarginBottom;

    mainTextBox.top = mainTextBoxTop - highlightedData.paddingBottom;

    const mainTextBoxLeft =
      canvas.width - mainTextBox.width - textMarginRight;

    mainTextBox.left = mainTextBoxLeft;

    canvas.renderAll();

    /* END Main text render */
  }

  /* BEGIN Contracted by render */

  if (options.contractedBy !== null) {
    contractedByTextbox = new fabric.Textbox(options.contractedBy, {
      left: contractedByTextSidesMargin,
      top: canvas.height - contractedByTextSidesMargin - contractedByTextSize,
      width: contractedByTextMaxWidth,
      fontFamily: "Roboto Condensed",
      fontSize: contractedByTextSize,
      textAlign: "left",
      fill: options.colors.contractedByText.value,
      selectable: false,
      zIndex: 20,
    });

    checkTextBoxHeight(contractedByTextbox, 1);

    canvas.add(contractedByTextbox);
  }

  /* END Contracted by render */

  /* BEGIN Main image render */

  if (
    options.mainImage !== null &&
    (!canvas.contains(mainImage) ||
      mainImage === null ||
      options.mainImage.src !== mainImageSource)
  ) {
    if (mainImage !== null) {
      canvas.remove(mainImage);
    }

    mainImage = new fabric.Image(options.mainImage, {
      left: 0,
      top: 0,
      zIndex: 0,
    });

    mainImage.controls = {
      ...fabric.Image.prototype.controls,
      mtr: new fabric.Control({ visible: false }),
    };

    if (mainImage.width >= mainImage.height) {
      mainImage.scaleToHeight(canvas.height);
    } else {
      mainImage.scaleToWidth(canvas.width);
    }

    canvas.add(mainImage);
    mainImageSource = options.mainImage.src;
    // canvas.centerObject(mainImage)
  } else if (mainImage !== null && options.mainImage === null) {
    canvas.remove(mainImage);
  }

  /* END Main image render */

  sortObjects(canvas);

  canvasPointerDownEvent = (event) => {
    let activeObject = canvas.getActiveObject();

    if (activeObject === null) {
      return;
    }

    // if (activeObject._element.src == mainImage._element.src) {
    // return
    // }

    canvas.remove(backgroundImage);
    backgroundImage = null;
  };

  if (!pointerDownEventAssigned) {
    document
      .getElementsByClassName("upper-canvas")[0]
      .addEventListener("pointerdown", canvasPointerDownEvent);

    pointerDownEventAssigned = true;
  }

  const colors = {...options.colors};

  upEventFunction = (event) => {
    redraw(canvas, {...options, colors: colors});
  };

  document
    .getElementsByClassName("upper-canvas")[0]
    .addEventListener("pointerup", upEventFunction);

  document
    .getElementsByClassName("upper-canvas")[0]
    .addEventListener("pointerout", upEventFunction);

  document
    .getElementsByClassName("upper-canvas")[0]
    .addEventListener("pointercancel", upEventFunction);
};

export default redraw;
